'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _glob = require('glob');

var _glob2 = _interopRequireDefault(_glob);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _rimraf = require('rimraf');

var _rimraf2 = _interopRequireDefault(_rimraf);

var _pify = require('pify');

var _pify2 = _interopRequireDefault(_pify);

var _pLimit = require('p-limit');

var _pLimit2 = _interopRequireDefault(_pLimit);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// We need to escape \ when used with constructor
// See: https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/RegExp
var CommentStartTpl = '(?:\\/\\/\\s|\\/\\*\\s?|<!--\\s?)';
var CommentEndTpl = '.*?\\n';

var getRegEx = function getRegEx(entity) {
  return ` *?${CommentStartTpl}${entity}_START${CommentEndTpl}((.|\n|\r)*?) *${CommentStartTpl}${entity}_END${CommentEndTpl}`;
};

var REGEX = {
  final: new RegExp(getRegEx('FINAL'), 'g'),
  workshop: new RegExp(getRegEx('WORKSHOP'), 'g'),
  comment: new RegExp(getRegEx('COMMENT'), 'g')
};
var openFileLimit = (0, _pLimit2.default)(100);

exports.default = splitGuide;


function splitGuide() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      templatesDir = _ref.templatesDir,
      exercisesDir = _ref.exercisesDir,
      exercisesFinalDir = _ref.exercisesFinalDir,
      clean = _ref.clean,
      ignore = _ref.ignore;

  return deletePreviouslyGeneratedFiles().then(getFiles).then(readAllFilesAsPromise).then(createNewFileContents).then(saveFiles);

  function getFiles() {
    var filesGlob = _path2.default.join(templatesDir, '**', '*');
    var globOptions = { nodir: true, ignore, dot: true };
    return (0, _pify2.default)(_glob2.default)(filesGlob, globOptions);
  }

  function deletePreviouslyGeneratedFiles() {
    if (!clean) {
      return _promise2.default.resolve();
    }
    var pRimraf = (0, _pify2.default)(_rimraf2.default);
    var opts = { disableGlob: true };
    return _promise2.default.all([pRimraf(exercisesDir, opts), pRimraf(exercisesFinalDir, opts)]);
  }

  function readFileAsPromise(file) {
    return (0, _pify2.default)(_fs2.default.readFile)(file, 'utf8').then(function (contents) {
      return { file, contents };
    });
  }

  function readAllFilesAsPromise(files) {
    var allPromises = files.map(function (file) {
      return openFileLimit(function () {
        return readFileAsPromise(file);
      });
    });
    return _promise2.default.all(allPromises);
  }

  function createNewFileContents(fileObjs) {
    return fileObjs.map(function (fileObj) {
      return (0, _assign2.default)({
        finalContents: createFinalContents(fileObj.contents),
        workshopContents: createWorkshopContents(fileObj.contents)
      }, fileObj);
    });
  }

  function createFinalContents(contents) {
    return contents.replace(REGEX.final, '$1').replace(REGEX.workshop, '').replace(REGEX.comment, '');
  }

  function createWorkshopContents(contents) {
    return contents.replace(REGEX.workshop, '$1').replace(REGEX.final, '').replace(REGEX.comment, '');
  }

  function saveFiles(fileObjs) {
    var allPromises = fileObjs.reduce(function (all, fileObj) {
      return [].concat((0, _toConsumableArray3.default)(all), (0, _toConsumableArray3.default)(saveFinalAndWorkshop(fileObj)));
    }, []);
    return _promise2.default.all(allPromises);
  }

  function saveFinalAndWorkshop(_ref2) {
    var file = _ref2.file,
        workshopContents = _ref2.workshopContents,
        finalContents = _ref2.finalContents;

    var relativeDestination = _path2.default.relative(templatesDir, file);
    var workshopDestination = _path2.default.resolve(exercisesDir, relativeDestination);
    var finalDestination = _path2.default.resolve(exercisesFinalDir, relativeDestination);
    return [workshopContents ? openFileLimit(function () {
      return saveFile(workshopDestination, workshopContents);
    }) : null, finalContents ? openFileLimit(function () {
      return saveFile(finalDestination, finalContents);
    }) : null].filter(Boolean); // filter out the files that weren't saved
  }

  function saveFile(file, contents) {
    return (0, _pify2.default)(_mkdirp2.default)(_path2.default.dirname(file), {}).then(function () {
      return (0, _pify2.default)(_fs2.default.writeFile)(file, contents).then(function () {
        return file;
      });
    }, (0, _utils.getErrorLogger)(`mkdirp(${_path2.default.dirname(file)})`));
  }
}